package djmil.cordacheckers.cordaclient;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatNoException;

import java.util.Arrays;
import java.util.Map;

import javax.naming.InvalidNameException;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import djmil.cordacheckers.cordaclient.dao.GameView;
import djmil.cordacheckers.cordaclient.dao.Rank;
import djmil.cordacheckers.cordaclient.dao.Stone;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.RspFailureException;
import djmil.cordacheckers.user.HoldingIdentityResolver;

@SpringBootTest
public class RankingTests {
    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    final String player1 = "Kumar";
    final String player2 = "Bobik";

    final static Stone WHITE_MAN  = new Stone(Stone.Color.WHITE, Stone.Type.MAN);
    final static Stone WHITE_KING = new Stone(Stone.Color.WHITE, Stone.Type.KING);
    final static Stone BLACK_MAN  = new Stone(Stone.Color.BLACK, Stone.Type.MAN);
    final static Stone BLACK_KING = new Stone(Stone.Color.BLACK, Stone.Type.KING);

    final static Map<Integer, Stone> board1 =  Map.ofEntries(
        Map.entry(10, BLACK_KING),
        Map.entry(7, WHITE_KING)
    );

    @Test
    void testSurrender() throws InvalidNameException, RspFailureException {
        final var hiCustodian  = holdingIdentityResolver.getCustodian();
        assertThat(hiCustodian).isNotNull();

        final Map<String, Rank> liderboard1 = cordaClient.fetchRanking(hiCustodian);

        final var hiWinner = holdingIdentityResolver.getByUsername(player1);
        final var hiLooser = holdingIdentityResolver.getByUsername(player2);
        final var winnerName = hiWinner.getName();
        final var losserName = hiLooser.getName();

        final GameView game = cordaClient.gameProposalCreate(hiWinner, hiLooser, Stone.Color.WHITE, 
            "GameBoard GLOBAL_RANKING surrender test");

        cordaClient.gameProposalAccept(hiLooser, game.uuid());
        cordaClient.gameBoardSurrender(hiLooser, game.uuid());

        final Map<String, Rank> liderboard2 = cordaClient.fetchRanking(hiCustodian);

        assertThat(liderboard1.get(winnerName).victory() +1 == liderboard2.get(winnerName).victory());
        assertThat(liderboard1.get(winnerName).total()   +1 == liderboard2.get(winnerName).total()  );
        assertThat(liderboard1.get(winnerName).draw()       == liderboard2.get(winnerName).draw()   );

        assertThat(liderboard1.get(losserName).victory()    == liderboard2.get(losserName).victory());
        assertThat(liderboard1.get(losserName).total()   +1 == liderboard2.get(losserName).total()  );
        assertThat(liderboard1.get(losserName).draw()       == liderboard2.get(losserName).draw()   );
    }

    @Test
    void testVictory() throws InvalidNameException, RspFailureException {
        final var hiCustodian  = holdingIdentityResolver.getCustodian();
        assertThat(hiCustodian).isNotNull();

        final Map<String, Rank> liderboard1 = cordaClient.fetchRanking(hiCustodian);

        final var hiWinner = holdingIdentityResolver.getByUsername(player2);
        final var hiLooser = holdingIdentityResolver.getByUsername(player1);
        final var winnerName = hiWinner.getName();
        final var losserName = hiLooser.getName();

        final GameView game = cordaClient.gameProposalCreate(hiWinner, hiLooser, Stone.Color.BLACK, 
            board1, "GameBoard GLOBAL_RANKING victory test");

        cordaClient.gameProposalAccept(hiLooser, game.uuid());
        cordaClient.gameBoardMove(hiWinner, game.uuid(), Arrays.asList(7, 14), null);

        final Map<String, Rank> liderboard2 = cordaClient.fetchRanking(hiCustodian);

        assertThat(liderboard1.get(winnerName).victory() +1 == liderboard2.get(winnerName).victory());
        assertThat(liderboard1.get(winnerName).total()   +1 == liderboard2.get(winnerName).total()  );
        assertThat(liderboard1.get(winnerName).draw()       == liderboard2.get(winnerName).draw()   );

        assertThat(liderboard1.get(losserName).victory()    == liderboard2.get(losserName).victory());
        assertThat(liderboard1.get(losserName).total()   +1 == liderboard2.get(losserName).total()  );
        assertThat(liderboard1.get(losserName).draw()       == liderboard2.get(losserName).draw()   );
    }

    @Test
    void testDraw() throws InvalidNameException, RspFailureException {
        final var hiCustodian  = holdingIdentityResolver.getCustodian();
        assertThat(hiCustodian).isNotNull();

        final var hiPlayer1 = holdingIdentityResolver.getByUsername(player1);
        final var hiPlayer2 = holdingIdentityResolver.getByUsername(player2);

        final GameView game = cordaClient.gameProposalCreate(hiPlayer1, hiPlayer2, Stone.Color.WHITE, 
            "GameBoard GLOBAL_RANKING draw test");

        cordaClient.gameProposalAccept(hiPlayer2, game.uuid());
        cordaClient.gameDrawRequest(hiPlayer2, game.uuid());

        final Map<String, Rank> liderboard1 = cordaClient.fetchRanking(hiCustodian);
        cordaClient.gameDrawAccept(hiPlayer1, game.uuid());
        final Map<String, Rank> liderboard2 = cordaClient.fetchRanking(hiCustodian);

        assertThat(liderboard1.get(player1).victory()  == liderboard2.get(player1).victory());
        assertThat(liderboard1.get(player1).total() +1 == liderboard2.get(player1).total()  );
        assertThat(liderboard1.get(player1).draw()  +1 == liderboard2.get(player1).draw()   );

        assertThat(liderboard1.get(player2).victory()  == liderboard2.get(player2).victory());
        assertThat(liderboard1.get(player2).total() +1 == liderboard2.get(player2).total()  );
        assertThat(liderboard1.get(player2).draw()  +1 == liderboard2.get(player2).draw()   );
    }

    @Test
    void testIndividualRanking() {
        final var hiCustodian  = holdingIdentityResolver.getByUsername(player2);

        assertThatNoException().isThrownBy( () -> 
            cordaClient.fetchRanking(hiCustodian)
        );
    }

}
