package djmil.cordacheckers.gamestate;

import java.security.PublicKey;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.InitiatingFlow;
import net.corda.v5.application.flows.SubFlow;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.application.messaging.FlowMessaging;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;

import static java.util.Objects.*;
import static java.util.stream.Collectors.toList;

@InitiatingFlow(protocol = "gamestate-commit")
public class CommitTrx implements SubFlow<SecureHash> {

    private final static Logger log = LoggerFactory.getLogger(CommitTrx.class);
    private final UtxoSignedTransaction utxTrxCandidate;
    private final List<PublicKey> participants;

    public CommitTrx(UtxoSignedTransaction signedTransaction, List<PublicKey> participants) {
        this.utxTrxCandidate = signedTransaction;
        this.participants = participants;
    }

    @CordaInject
    public UtxoLedgerService ledgerService;

    @CordaInject
    public FlowMessaging flowMessaging;

    @CordaInject
    public MemberLookup memberLookup;

    @Override
    @Suspendable
    public SecureHash call() {
        log.info("GameState commit started");

        final MemberX500Name myName = memberLookup.myInfo().getName();

        List<FlowSession> sessions = participants.stream()
                .map(pubKey -> requireNonNull(memberLookup.lookup(pubKey), "Member not found from public Key " + pubKey + ".").getName())
                .filter(person -> person.compareTo(myName) != 0)
                .map(signatorieX500name -> flowMessaging.initiateFlow(signatorieX500name))
                .collect(toList());

        if (sessions.size() != 2)
            throw new RuntimeException("Should be strictly TWO signatories other than the initiator");

        /*
         * Calls the Corda provided finalise() function which gather signatures from the
         * counterparty,
         * notarises the transaction and persists the transaction to each party's vault.
         */

        final SecureHash trxId = ledgerService
                .finalize(this.utxTrxCandidate, sessions)
                .getTransaction()
                .getId();

        log.info("GameState commit " + trxId);
        return trxId;
    }
}
